@@ -18,7 +18,7 @@
 ''' This module represents a bakup. '''
 
 from barman import xlog, _pretty_size
-from barman.command_wrappers import RsyncPgData
+from barman.command_wrappers import RsyncPgData, Command
 from barman.compression import CompressionManager, CompressionIncompatibility
 from glob import glob
 import ast
@@ -149,8 +149,8 @@ class BackupInfo(object):
                 yield "  PGDATA directory  : %s" % self.pgdata
                 if self.tablespaces:
                     yield "  Tablespaces:"
-                    for name, _, location in self.tablespaces:
-                        yield "    %s: %s" % (name, location)
+                    for name, oid, location in self.tablespaces:
+                        yield "    %s: %s (oid: %s)" % (name, location, oid)
                 yield ""
                 yield "  Base backup information:"
                 yield "    Disk usage      : %s" % _pretty_size(self.size + wal_size)
@@ -232,7 +232,10 @@ class BackupManager(object):
     '''Manager of the backup archive for a server'''
 
     DEFAULT_STATUS_FILTER = (BackupInfo.DONE,)
-
+    DANGEROUS_OPTIONS = ['data_directory', 'config_file', 'hba_file',
+            'ident_file', 'external_pid_file', 'ssl_cert_file',
+            'ssl_key_file', 'ssl_ca_file', 'ssl_crl_file',
+            'unix_socket_directory']
     def __init__(self, server):
         '''Constructor'''
         self.name = "default"
@@ -240,6 +243,9 @@ class BackupManager(object):
         self.config = server.config
         self.available_backups = {}
         self.compression_manager = CompressionManager(self.config)
+        
+        # used for error messages
+        self.current_action = None
 
     def get_available_backups(self, status_filter=DEFAULT_STATUS_FILTER):
         '''
@@ -364,13 +370,66 @@ class BackupManager(object):
                 os.rename(xlogdb_new, fxlogdb.name)
         yield "Done"
 
+
+    def build_script_env(self, backup_info, phase):
+        """
+        Prepare the environment for executing a script
+        """
+        previous_backup = self.get_previous_backup(backup_info.backup_id)
+        env = {}
+        env['BARMAN_BACKUP_DIR'] = backup_info.get_basebackup_directory()
+        env['BARMAN_SERVER'] = self.config.name
+        env['BARMAN_CONFIGURATION'] = self.config.config.config_file
+        env['BARMAN_BACKUP_ID'] = backup_info.backup_id
+        env['BARMAN_PREVIOUS_ID'] =  previous_backup.backup_id if previous_backup else ''
+        env['BARMAN_PHASE'] = phase
+        env['BARMAN_STATUS'] = backup_info.status
+        env['BARMAN_ERROR'] = backup_info.error or ''
+        return env
+
+    def run_pre_backup_script(self, backup_info):
+        '''
+        Run the pre_backup_script if configured.
+        This method must never throw any exception
+        '''
+        try:
+            script = self.config.pre_backup_script
+            if script:
+                _logger.info("Attempt to run pre_backup_script: %s", script)
+                cmd = Command(
+                    script,
+                    env_append=self.build_script_env(backup_info, 'pre'),
+                    shell=True, check=False)
+                ret = cmd()
+                _logger.info("pre_backup_script returned %d", ret)
+        except Exception:
+            _logger.exception('Exception running pre_backup_script')
+
+    def run_post_backup_script(self, backup_info):
+        '''
+        Run the post_backup_script if configured.
+        This method must never throw any exception
+        '''
+        try:
+            script = self.config.post_backup_script
+            if script:
+                _logger.info("Attempt to run post_backup_script: %s", script)
+                cmd = Command(
+                    script,
+                    env_append=self.build_script_env(backup_info, 'post'),
+                    shell=True, check=False)
+                ret = cmd()
+                _logger.info("post_backup_script returned %d", ret)
+        except Exception:
+            _logger.exception('Exception running post_backup_script')
+
     def backup(self):
         '''
         Performs a backup for the server
         '''
         _logger.debug("initialising backup information")
         backup_stamp = datetime.datetime.now()
-        current_action = "starting backup"
+        self.current_action = "starting backup"
         backup_info = None
         try:
             backup_info = BackupInfo(self.server, backup_id=backup_stamp.strftime('%Y%m%dT%H%M%S'))
@@ -380,6 +439,9 @@ class BackupManager(object):
             _logger.info(msg)
             yield msg
 
+            # Run the pre-backup-script if present.
+            self.run_pre_backup_script(backup_info)
+
             # Start the backup
             self.backup_start(backup_info)
             backup_info.set_attribute("begin_time", backup_stamp)
@@ -389,8 +451,8 @@ class BackupManager(object):
             yield msg
             _logger.info(msg)
 
-            current_action = "copying files"
-            _logger.debug(current_action)
+            self.current_action = "copying files"
+            _logger.debug(self.current_action)
             try:
                 # Start the copy
                 msg = "Copying files."
@@ -404,7 +466,7 @@ class BackupManager(object):
             except:
                 raise
             else:
-                current_action = "issuing stop of the backup"
+                self.current_action = "issuing stop of the backup"
                 msg = "Asking PostgreSQL server to finalize the backup."
                 yield msg
                 _logger.info(msg)
@@ -416,9 +478,9 @@ class BackupManager(object):
         except:
             if backup_info:
                 backup_info.set_attribute("status", "FAILED")
-                backup_info.set_attribute("error", "failure %s" % current_action)
+                backup_info.set_attribute("error", "failure %s" % self.current_action)
 
-            msg = "Backup failed %s" % current_action
+            msg = "Backup failed %s" % self.current_action
             _logger.exception(msg)
             yield msg
 
@@ -434,6 +496,10 @@ class BackupManager(object):
             if backup_info:
                 backup_info.save()
 
+            # Run the post-backup-script if present.
+            self.run_post_backup_script(backup_info)
+
+
     def recover(self, backup, dest, tablespaces, target_tli, target_time, target_xid, exclusive, remote_command):
         '''
         Performs a recovery of a backup
@@ -600,16 +666,32 @@ class BackupManager(object):
             msg = "The archive_command was set to 'false' to prevent data losses."
             yield msg
             _logger.info(msg)
+
+        # Find dangerous options in the configuration file (locations)
+        clashes = self.pg_config_detect_possible_issues(pg_config)
+
         if remote_command:
             recovery = rsync.from_file_list(['postgresql.conf', 'postgresql.conf.origin'], tempdir, ':%s' % dest)
             shutil.rmtree(tempdir)
 
-        yield "Restore done!"
+
         yield ""
-        yield "Please review the network and archiving related settings"
-        yield "in the postgres configuration file before start the just recovered instance."
+        yield "Your PostgreSQL server has been successfully prepared for recovery!"
         yield ""
-        _logger.info("Restore completed successful.")
+        yield "Please review network and archive related settings in the PostgreSQL"
+        yield "configuration file before starting the just recovered instance."
+        yield ""
+        if clashes:
+            yield "WARNING: Before starting up the recovered PostgreSQL server,"
+            yield "please review also the settings of the following configuration"
+            yield "options as they might interfere with your current recovery attempt:"
+            yield ""
+
+            for name, value in sorted(clashes.items()):
+                yield "    %s = %s" % (name, value)
+
+            yield ""
+        _logger.info("Recovery completed successful.")
 
 
     def cron(self, verbose):
@@ -665,11 +747,15 @@ class BackupManager(object):
         :param name: the name of the WAL to delete
         '''
         hashdir = os.path.join(self.config.wals_directory, xlog.hash_dir(name))
-        os.unlink(os.path.join(hashdir, name))
         try:
-            os.removedirs(hashdir)
+            os.unlink(os.path.join(hashdir, name))
+            try:
+                os.removedirs(hashdir)
+            except:
+                pass
         except:
-            pass
+            _logger.warning('Expected WAL file %s not found during delete',
+                name)
 
     def backup_start(self, backup_info):
         '''
@@ -677,12 +763,12 @@ class BackupManager(object):
 
         :param backup_info: the backup information structure
         '''
-        current_action = "connecting to database (%s)" % self.config.conninfo
-        _logger.debug(current_action)
+        self.current_action = "connecting to database (%s)" % self.config.conninfo
+        _logger.debug(self.current_action)
 
         # Set the PostgreSQL data directory
-        current_action = "detecting data directory"
-        _logger.debug(current_action)
+        self.current_action = "detecting data directory"
+        _logger.debug(self.current_action)
         data_directory = self.server.get_pg_setting('data_directory')
         backup_info.set_attribute('pgdata', data_directory)
 
@@ -696,8 +782,8 @@ class BackupManager(object):
                 backup_info.set_attribute(key, cf[key])
 
         # Get server version and tablespaces information
-        current_action = "detecting tablespaces"
-        _logger.debug(current_action)
+        self.current_action = "detecting tablespaces"
+        _logger.debug(self.current_action)
         tablespaces = self.server.get_pg_tablespaces()
         if tablespaces and len(tablespaces) > 0:
             backup_info.set_attribute("tablespaces", tablespaces)
@@ -706,16 +792,19 @@ class BackupManager(object):
                 _logger.info(msg)
 
         # Issue pg_start_backup on the PostgreSQL server
-        current_action = "issuing pg_start_backup command"
-        _logger.debug(current_action)
-        start_xlog, start_file_name, start_file_offset = self.server.pg_start_backup()
-        msg = "Backup begin at xlog location: %s (%s, %08X)" % (start_xlog, start_file_name, start_file_offset)
-        _logger.info(msg)
-        backup_info.set_attribute("status", "STARTED")
-        backup_info.set_attribute("timeline", int(start_file_name[0:8]))
-        backup_info.set_attribute("begin_xlog", start_xlog)
-        backup_info.set_attribute("begin_wal", start_file_name)
-        backup_info.set_attribute("begin_offset", start_file_offset)
+        self.current_action = "issuing pg_start_backup command"
+        _logger.debug(self.current_action)
+        start_row = self.server.pg_start_backup()
+        if start_row:
+            start_xlog, start_file_name, start_file_offset = start_row
+            backup_info.set_attribute("status", "STARTED")
+            backup_info.set_attribute("timeline", int(start_file_name[0:8], 16))
+            backup_info.set_attribute("begin_xlog", start_xlog)
+            backup_info.set_attribute("begin_wal", start_file_name)
+            backup_info.set_attribute("begin_offset", start_file_offset)
+        else:
+            self.current_action = "starting the backup: PostgreSQL server is already in exclusive backup mode"
+            raise Exception('concurrent exclusive backups are not allowed')
 
     def backup_copy(self, backup_info):
         '''
@@ -733,26 +822,26 @@ class BackupManager(object):
             raise Exception(msg)
 
         # Copy configuration files (if not inside PGDATA)
-        current_action = "copying configuration files"
-        _logger.debug(current_action)
+        self.current_action = "copying configuration files"
+        _logger.debug(self.current_action)
         cf = self.server.get_pg_configuration_files()
         if cf:
             for key in sorted(cf.keys()):
                 # Consider only those that reside outside of the original PGDATA
                 if cf[key]:
                     if cf[key].find(backup_info.pgdata) == 0:
-                        current_action = "skipping %s as contained in %s directory" % (key, backup_info.pgdata)
-                        _logger.debug(current_action)
+                        self.current_action = "skipping %s as contained in %s directory" % (key, backup_info.pgdata)
+                        _logger.debug(self.current_action)
                         continue
                     else:
-                        current_action = "copying %s as outside %s directory" % (key, backup_info.pgdata)
-                        _logger.info(current_action)
+                        self.current_action = "copying %s as outside %s directory" % (key, backup_info.pgdata)
+                        _logger.info(self.current_action)
                         retval = rsync(':%s' % cf[key], backup_dest)
                         if retval not in (0, 24):
                             raise Exception("ERROR: data transfer failure")
 
-        current_action = "calculating backup size"
-        _logger.debug(current_action)
+        self.current_action = "calculating backup size"
+        _logger.debug(self.current_action)
         backup_size = 0
         for dirpath, _, filenames in os.walk(backup_dest):
             for f in filenames:
@@ -915,3 +1004,26 @@ class BackupManager(object):
                 f.write(line)
 
         return mangled
+
+    def pg_config_detect_possible_issues(self, filename):
+        '''This method looks for any possible issue with PostgreSQL
+        location options such as data_directory, config_file, etc.
+        It returns a dictionary with the dangerous options that have been found.
+
+        :param filename: the Postgres configuration file
+        '''
+
+        clashes = {}
+
+        with open(filename) as f:
+            content = f.readlines()
+
+        r = re.compile('^\s*([^\s=]+)\s*=\s*(.*)$')
+        for line in content:
+            rm = r.match(line)
+            if rm:
+                key = rm.group(1)
+                if key in self.DANGEROUS_OPTIONS:
+                    clashes[key] = rm.group(2)
+
+        return clashes
