@@ -26,8 +26,6 @@ import barman.config
 import logging
 import os
 import sys
-import textwrap
-from argparse import RawDescriptionHelpFormatter
 from barman.backup import BackupInfo
 
 _logger = logging.getLogger(__name__)
@@ -37,12 +35,12 @@ _logger = logging.getLogger(__name__)
 def list_server(args):
     """ List available servers, with useful information
     """
-    servers = barman.__config__.servers()
-    for server in servers:
+    for name in sorted(barman.__config__.server_names()):
+        server = barman.__config__.get_server(name)
         if server.description and not args.minimal:
-            yield "%s - %s" % (server.name, server.description)
+            yield "%s - %s" % (name, server.description)
         else:
-            yield server.name
+            yield name
 
 def cron(args):
     """ Run maintenance tasks
@@ -58,41 +56,46 @@ def cron(args):
                 for lines in server.cron(verbose=True):
                     yield lines
 
-@arg('server_name', help='specifies the server name for the command')
+@arg('server_name', nargs='+', help="specifies the server names for the backup command ('all' will show all available servers)")
 def backup(args):
     """ Perform a full backup for the given server
     """
-    server = get_server(args)
-    if server == None:
-        yield "Unknown server '%s'" % (args.server_name)
-        return
-    for line in server.backup():
-        yield line
+    servers = get_server_list(args)
+    for name in sorted(servers):
+        server = servers[name]
+        if server == None:
+            yield "Unknown server '%s'" % (name)
+            continue
+        for line in server.backup():
+            yield line
+        yield ''
 
 @alias('list-backup')
-@arg('server_name', help='specifies the server name for the command')
+@arg('server_name', nargs='+', help="specifies the server name for the command ('all' will show all available servers)")
 @arg('--minimal', help='machine readable output', action='store_true')
 def list_backup(args):
-    """ List available backups for the given server
+    """ List available backups for the given server (supports 'all')
     """
-    server = get_server(args)
-    if server == None:
-        yield "Unknown server '%s'" % (args.server_name)
-        return
-    if not args.minimal:
-        for line in server.list_backups():
-            yield line
-    else:
-        for backup_id in sorted(server.get_available_backups().iterkeys(), reverse=True):
-            yield backup_id
+    servers = get_server_list(args)
+    for name in sorted(servers):
+        server = servers[name]
+        if server == None:
+            yield "Unknown server '%s'" % (name)
+            return
+        if not args.minimal:
+            for line in server.list_backups():
+                yield line
+        else:
+            for backup_id in sorted(server.get_available_backups().iterkeys(), reverse=True):
+                yield backup_id
 
 @arg('server_name', nargs='+', help='specifies the server name for the command')
 def status(args):
     """ Shows live information and status of the PostgreSQL server
     """
     servers = get_server_list(args)
-
-    for name, server in servers.items():
+    for name in sorted(servers):
+        server = servers[name]
         if server == None:
             yield "Unknown server '%s'" % (name)
             continue
@@ -125,18 +128,33 @@ def recover(args):
     """
     server = get_server(args)
     if server == None:
-        raise SystemExit("ERROR: Unknown server '%s'" % (args.server_name))
+        raise SystemExit("ERROR: unknown server '%s'" % (args.server_name))
     # Retrieves the backup info
-    backup = server.get_backup(args.backup_id)
+    backup_id = parse_backup_id(server, args)
+    backup = server.get_backup(backup_id)
     if backup == None or backup.status != BackupInfo.DONE:
-        raise SystemExit("ERROR: Unknown backup '%s' for server '%s'" % (args.backup_id, args.server_name))
+        raise SystemExit("ERROR: unknown backup '%s' for server '%s'" % (args.backup_id, args.server_name))
+    # decode the tablespace relocation rules
     tablespaces = {}
     if args.tablespace:
         for rule in args.tablespace:
             try:
                 tablespaces.update([rule.split(':', 1)])
             except:
-                yield "WARNING: Invalid tablespace relocation rule %s" % rule
+                raise SystemExit("ERROR: invalid tablespace relocation rule '%s'\n"
+                                 "HINT: the valid syntax for a relocation rule is NAME:LOCATION" % rule)
+    # validate the rules against the tablespace list
+    valid_tablespaces = [tablespace_data[0] for tablespace_data in backup.tablespaces] if backup.tablespaces else []
+    for tablespace in tablespaces:
+        if tablespace not in valid_tablespaces:
+            raise SystemExit("ERROR: invalid tablespace name '%s'\n"
+                             "HINT: please use any of the following tablespaces: %s" 
+                             % (tablespace, ', '.join(valid_tablespaces)))
+    # explicitly disallow the rsync remote syntax (common mistake)
+    if ':' in args.destination_directory:
+        raise SystemExit(
+            "ERROR: the destination directory parameter cannot contain the ':' character\n"
+            "HINT: if you want to do a remote recovery you have to use the --remote-ssh-command option")
     if args.remote_ssh_command and len(tablespaces) > 0:
         raise SystemExit("ERROR: Tablespace relocation is not supported with remote recovery")
     for line in server.recover(backup,
@@ -156,8 +174,8 @@ def show_server(args):
     """ Show all configuration parameters for the specified servers
     """
     servers = get_server_list(args)
-
-    for name, server in servers.items():
+    for name in sorted(servers):
+        server = servers[name]
         if server == None:
             yield "Unknown server '%s'" % (name)
             continue
@@ -168,12 +186,12 @@ def show_server(args):
 @arg('server_name', nargs='+', help="specifies the server names to check ('all' will check all available servers)")
 def check(args):
     """ Check if the server configuration is working.
-    Thsi function returns 0 if every checks pass, or 0 if any of these fails
+    This function returns 0 if every checks pass, or 0 if any of these fails
     """
     servers = get_server_list(args)
-
     ok = True
-    for name, server in servers.items():
+    for name in sorted(servers):
+        server = servers[name]
         if server == None:
             yield "Unknown server '%s'" % (name)
             ok = False
@@ -182,8 +200,8 @@ def check(args):
             ok &= status
             yield line
         yield ''
-        if not ok:
-            raise SystemExit(1)
+    if not ok:
+        raise SystemExit(1)
 
 @alias('show-backup')
 @arg('server_name', help='specifies the server name for the command')
@@ -196,7 +214,8 @@ def show_backup(args):
         yield "Unknown server '%s'" % (args.server_name)
         return
     # Retrieves the backup info
-    backup = server.get_backup(args.backup_id)
+    backup_id = parse_backup_id(server, args)
+    backup = server.get_backup(backup_id)
     if backup == None:
         yield "Unknown backup '%s' for server '%s'" % (args.backup_id, args.server_name)
         return
@@ -221,7 +240,8 @@ def list_files(args):
         yield "Unknown server '%s'" % (args.server_name)
         return
     # Retrieves the backup info
-    backup = server.get_backup(args.backup_id)
+    backup_id = parse_backup_id(server, args)
+    backup = server.get_backup(backup_id)
     if backup == None:
         yield "Unknown backup '%s' for server '%s'" % (args.backup_id, args.server_name)
         return
@@ -238,7 +258,8 @@ def delete(args):
         yield "Unknown server '%s'" % (args.server_name)
         return
     # Retrieves the backup info
-    backup = server.get_backup(args.backup_id)
+    backup_id = parse_backup_id(server, args)
+    backup = server.get_backup(backup_id)
     if backup == None:
         yield "Unknown backup '%s' for server '%s'" % (args.backup_id, args.server_name)
         return
@@ -300,6 +321,19 @@ def get_server_list(args):
                 server_dict[server] = Server(conf)
         return server_dict
 
+def parse_backup_id(server, args):
+    ''' Parses special backup IDs such as latest, oldest, etc. '''
+    if args.backup_id in ('latest', 'last'):
+        backup_id = server.get_last_backup()
+    elif args.backup_id in ('oldest', 'first'):
+        backup_id = server.get_first_backup()
+    else:
+        return args.backup_id
+    if backup_id == None:
+        raise SystemExit("ERROR: '%s' backup is not available for server '%s'" % (args.backup_id, args.server_name))
+    return backup_id
+
+
 def main():
     ''' The main method of Barman '''
     p = ArghParser()
