@@ -25,6 +25,7 @@ import pwd
 import grp
 from ConfigParser import ConfigParser, NoOptionError
 import logging.handlers
+from glob import iglob
 
 _logger = logging.getLogger(__name__)
 
@@ -34,12 +35,15 @@ class Server(object):
     KEYS = ['active', 'description', 'ssh_command', 'conninfo',
         'backup_directory', 'basebackups_directory',
         'wals_directory', 'incoming_wals_directory', 'lock_file',
-        'compression', 'custom_compression_filter', 'custom_decompression_filter',
-        'retention_policy', 'wal_retention_policy',
+        'compression', 'custom_compression_filter',
+        'custom_decompression_filter', 'retention_policy',
+        'wal_retention_policy', 'pre_backup_script', 'post_backup_script',
     ]
 
-    BARMAN_KEYS = ['compression', 'custom_compression_filter', 'custom_decompression_filter',
-        'retention_policy', 'wal_retention_policy',
+    BARMAN_KEYS = ['compression', 'custom_compression_filter',
+        'custom_decompression_filter', 'retention_policy',
+        'wal_retention_policy', 'pre_backup_script', 'post_backup_script',
+        'configuration_files_directory',
     ]
 
     DEFAULTS = {
@@ -52,6 +56,7 @@ class Server(object):
     }
 
     def __init__(self, config, name):
+        self.config = config
         self.name = name
         self.barman_home = config.get('barman', 'barman_home')
         for key in Server.KEYS:
@@ -87,7 +92,7 @@ class Config(object):
         self.config_file = filename
         self._servers = None
         self._parse_global_config()
-
+        self._load_configuration_files_directory()
 
     def get(self, section, option, defaults={}):
         '''Method to get the value from a given section from
@@ -139,6 +144,10 @@ class Config(object):
             logging.root.setLevel(level_int)
         else:
             _logger.warn('unknown log_level in config file: %s', level)
+        self._global_config = set(self._config.items('barman'))
+
+    def _is_global_config_changed(self):
+        return self._global_config != set(self._config.items('barman'))
 
     def _enforce_user(self):
         '''Set the correct user'''
@@ -160,6 +169,36 @@ class Config(object):
             raise SystemExit(msg)
         os.environ['HOME'] = pw.pw_dir
 
+    def _load_configuration_files_directory(self):
+        '''Read the "configuration_files_directory" option and loads all the
+        configuration files with the .conf suffix that lie in that folder
+        '''
+
+        configuration_files_directory = self.get('barman', 'configuration_files_directory')
+
+        if not configuration_files_directory:
+           return
+
+        if not os.path.isdir(os.path.expanduser(configuration_files_directory)):
+           _logger.warn('Ignoring the "configuration_files_directory" option as "%s" is not a directory',
+                configuration_files_directory)
+           return
+
+        for cfile in sorted(iglob(os.path.join(os.path.expanduser(configuration_files_directory),
+            '*.conf'))):
+           filename = os.path.basename(cfile)
+           if os.path.isfile(cfile):
+               # Load a file
+               _logger.info('Including configuration file: %s', filename)
+               self._config.read(cfile)
+               if self._is_global_config_changed():
+                   msg="the configuration file %s contains a not empty [barman] section" % filename
+                   _logger.fatal(msg)
+                   raise SystemExit("FATAL: %s" % msg)
+           else:
+               # Add an info that a file has been discarded
+               _logger.warn('Discarding configuration file: %s (not a file)', filename)
+
     def _populate_servers(self):
         '''Populate server list from configuration file'''
         if self._servers != None:
