@@ -80,8 +80,10 @@ class Server(object):
         error = None
         try:
             for key in self.config.KEYS:
-                if key.endswith('_directory') and hasattr(self.config, key) and not os.path.isdir(getattr(self.config, key)):
-                    os.makedirs(getattr(self.config, key))
+                if key.endswith('_directory') and hasattr(self.config, key):
+                    val = getattr(self.config, key)
+                    if val != None and not os.path.isdir(val):
+                        os.makedirs(val)
         except OSError, e:
                 error = e.strerror
         if not error:
@@ -196,6 +198,11 @@ class Server(object):
         if myconn:
             self.conn = psycopg2.connect(self.config.conninfo)
             self.server_version = self.conn.server_version
+            if (self.server_version >= 90000
+                and 'application_name=' not in self.config.conninfo):
+                cur = self.conn.cursor()
+                cur.execute('SET application_name TO barman')
+                cur.close()
         try:
             yield self.conn
         finally:
@@ -221,8 +228,8 @@ class Server(object):
         with self.pg_connect() as conn:
             try:
                 cur = conn.cursor()
-                if self.server_version > 90200:
-                    cur.execute("SELECT spcname, oid, pg_tablespace_location(oid) AS spclocation FROM pg_tablespace WHERE spclocation != ''")
+                if self.server_version >= 90200:
+                    cur.execute("SELECT spcname, oid, pg_tablespace_location(oid) AS spclocation FROM pg_tablespace WHERE pg_tablespace_location(oid) != ''")
                 else:
                     cur.execute("SELECT spcname, oid, spclocation FROM pg_tablespace WHERE spclocation != ''")
                 return cur.fetchall()
@@ -282,6 +289,22 @@ class Server(object):
         '''
         return self.backup_manager.get_available_backups(status_filter)
 
+    def get_last_backup(self, status_filter=BackupManager.DEFAULT_STATUS_FILTER):
+        '''
+        Get the last backup (if any) in the catalog
+
+        :param status_filter: default DEFAULT_STATUS_FILTER. The status of the backup returned
+        '''
+        return self.backup_manager.get_last_backup(status_filter)
+
+    def get_first_backup(self, status_filter=BackupManager.DEFAULT_STATUS_FILTER):
+        '''
+        Get the first backup (if any) in the catalog
+
+        :param status_filter: default DEFAULT_STATUS_FILTER. The status of the backup returned
+        '''
+        return self.backup_manager.get_first_backup(status_filter)
+
     def list_backups(self):
         '''Lists all the available backups for the server'''
         status_filter = BackupInfo.STATUS_NOT_EMPTY
@@ -444,5 +467,8 @@ class Server(object):
         except ValueError:
             # Old format compatibility (no compression)
             compression = None
-            name, size, stamp = line.split()
+            try:
+                name, size, stamp = line.split()
+            except:
+                raise ValueError("cannot parse line: %r" % (line,))
         return name, int(size), float(stamp), compression
